/***************************************************************************
 *   Copyright (C) 2009 by Eugene Korotkov   *
 *   cyberozz@yandex.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "unistd.h"
#include "getopt.h"
#include "dev.h"
#include "sleep.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <errno.h>

#define MAX_TRIES 10
int theVerbosity = 0;

int help(char *argv[]) {
       fprintf(stderr, "Usage: %s [-l lockfile] [-v level] [-d dev_name]\n"
    		       "	-l lockfile	file for device locking.\n"
		       "			By default /var/lock/the3g-tools/the3g-tools.lock\n"
		       "	-v level	debug level. 0 - no debug, 1 - debug.\n"
		       "			debug send his output to stderr\n"
		       "	-d dev_name	device name for monitoring. Default /dev/ttyUSB1\n\n"
		       "%s produce \"radio type string GPRS/GSM/UMTS\" as output\n",
         argv[0]);
  return 0;
}

int work(char *dev_name, char *lock_file) {
  t_device dev;
  dev.fd = -1;
  dev.name = dev_name;
  dev.speed = 921600;

  int count = 0;
  DEBUG("Check device locked...\n");
  while (is_devlocked(lock_file) == 1) {
    DEBUG("Locked. Sleep some time and recheck.\n");
    mysleep(20000);
    count++;
    if (count > 100) {
      DEBUG("Something wrong. need to unlock file\n");
      unlock_dev(lock_file);
    }
  };
  DEBUG("Device unlocked.\n");

  DEBUG("Locking device...\n");
  if (!lock_dev(lock_file)) {
    DEBUG("Error locking device. String: %s\n", strerror(errno));
    return -1;
  }

  DEBUG("Open device...\n");
  dev.fd = open_dev(&dev, 200000);

  if (dev.fd != -1) {
    DEBUG("OK\n");

    DEBUG("Clear all IN buffers.\n");
    while(get_onebyte(&dev, 2000) > 0) {};

    char *str = calloc(1, 1024);

    //init modem
    DEBUG("Initizlizing modem.\n");

    DEBUG("Sent ATZ\n");
    write_dev(&dev, "ATZ\r");
    unsigned int tries = 0;
    do {
      memset(str, 0, 1024);
      str = get_string(&dev, str, 1024, 10000);
      if (strlen(str) > 0) {
        DEBUG("Got string: %s\n", str)
      }
      mysleep(200);
      tries++;
    } while (strcmp(str, "OK") && tries < MAX_TRIES);

    DEBUG("Clear all IN buffers.\n");
    while(get_onebyte(&dev, 2000) > 0) {};


    //get signal quality
    char *oper = 0;
    DEBUG("Sent AT+COPS?\n");

    write_dev(&dev, "AT+COPS?\r");
    tries = 0;
    do {
      memset(str, 0, 1024);
      str = get_string(&dev, str, 1024, 10000);
      if (strlen(str) > 0) {
        DEBUG("Got string: %s\n", str)
      }
      tries++;
      mysleep(200);
    } while (((oper = strstr(str, "+COPS:")) == 0) && tries < MAX_TRIES);
    
    if (oper > 0) {
      if (strlen(oper) > 14) {
        char op_name[128];
	memset(op_name, 0, 128);
        char *ch = strchr(oper, '\"');
        if (ch) {
  	  ch = strchr(ch+1, '\"');
	  if (ch) {
            ch = strchr(ch+1, ',');
	    if (ch) {
              int op = atoi(ch+1);
	      if (op == 0)
	        printf("GPRS\n");
	      else if (op == 1)
	        printf("GSM compact\n");
	      else if (op == 2)
	        printf("UMTS\n");
	      else
	        printf("UNKNOWN\n");
	    }
	  }
	}
      }
    }

    DEBUG("Clear all IN buffers.\n");
    while(get_onebyte(&dev, 2000) > 0) {};

    //resetting modem
    DEBUG("Resetting modem\n");
    DEBUG("Sent ATZ\n");
    write_dev(&dev, "ATZ\r");

    mysleep(10000);

    tries = 0;
    do {
      memset(str, 0, 1024);
      str = get_string(&dev, str, 1024, 10000);
      if (strlen(str) > 0) {
        DEBUG("Got string: %s\n", str)
      }
      mysleep(200);
      tries++;
    } while (strcmp(str, "OK") && tries < MAX_TRIES);

    while(get_onebyte(&dev, 2000) > 0) {};
  }
  else {
    DEBUG("ERROR. String: %s\n", strerror(errno));

    DEBUG("Unlocking device...\n");
    if (!unlock_dev(lock_file)) {
      DEBUG("Error unlocking device. String: %s\n", strerror(errno));
    }
    return -1;
  }

  DEBUG("Closing device\n");
  close_dev(&dev);

  DEBUG("Unlocking device...\n");
  if (!unlock_dev(lock_file)) {
    DEBUG("Error unlocking device. String: %s\n", strerror(errno));
    return -1;
  }

  return 0;
}

int main(unsigned int argc, char*argv[]) {
  int opt;

  char lock_file[1024];
  memset(lock_file, 0, sizeof(lock_file));
  //set default lock file name
  strcpy(lock_file,  "/var/lock/the3g-tools/the3g-tools.lock");

  char dev_name[1024];
  memset(dev_name, 0, sizeof(dev_name));
  //set default device name
  strcpy(dev_name, "/dev/ttyUSB1");

  while ((opt = getopt(argc, argv, "v:l:d:")) != -1) {
    switch (opt) {
     case 'l':
       memset(lock_file, 0, sizeof(1024));
       strcpy(lock_file,  argv[optind-1]);
       break;
     case 'd':
       memset(dev_name, 0, sizeof(dev_name));
       strcpy(dev_name, argv[optind-1]);
       break;
     case 'v':
       theVerbosity = atoi(argv[optind-1]);
       break;
     default: /* '?' */
       help(argv);
       return -1;
    }
  }

  return work(dev_name, lock_file);
}
