/***************************************************************************
 *   Copyright (C) 2009 by Eugene Korotkov   *
 *   cyberozz@yandex.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <asm/termbits.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/time.h>

#include <string.h>

#include "dev.h"
#include "sleep.h"

bool close_dev(t_device *dev)
{
  if (dev->fd != -1)
    close(dev->fd);
  dev->fd=-1;

  return true;
}

int open_dev(t_device *dev, unsigned long sleeptime) 
{
  dev->fd = -1;
  if ((dev->fd = open(dev->name, O_RDWR|O_EXCL|O_NONBLOCK|O_NOCTTY)) < 0) 
  {
    dev->fd = -1;
    return dev->fd;
  }

  struct stat dev_stat;
  if (fstat(dev->fd, &dev_stat) != 0)
  {
    dev->fd = -1;
    return dev->fd;
  }
  
  if (!S_ISCHR(dev_stat.st_mode))
  {
    return dev->fd;
  }

  struct termio dev_termio;
  if (ioctl (dev->fd, TCGETA, &dev_termio) < 0) 
  {
	dev->fd = -1;
	return dev->fd;
  }
  
  dev_termio.c_iflag &= ~(IGNCR | ICRNL | IGNBRK | IUCLC | INPCK | IXON | IXANY | IGNPAR );
  dev_termio.c_oflag &= ~(OPOST | OLCUC | OCRNL | ONLCR | ONLRET);
  dev_termio.c_lflag &= ~(ICANON | XCASE | ECHO | ECHOE | ECHONL);
  dev_termio.c_lflag |= ISIG | IEXTEN | ECHOK | ECHOKE | ECHOCTL;
  dev_termio.c_cc[VMIN] = 1;
  dev_termio.c_cc[VTIME] = 5;
  dev_termio.c_cc[VEOF] = 1;

  dev_termio.c_cflag &= ~(CBAUD | CSIZE | CSTOPB | CLOCAL | PARENB);
  dev_termio.c_cflag |= CLOCAL | HUPCL;
  dev_termio.c_cflag |= (dev->speed | CS8 | CREAD );
  if (ioctl(dev->fd, TCSETA, &dev_termio) < 0) 
  {
    dev->fd = -1;
    return dev->fd;
  }

  mysleep(200000L);

  return dev->fd;
}

bool write_dev(t_device *dev, const char *string) 
{
  if (dev->fd == -1)
  {
    return false;
  }
	
  for(unsigned int i=0; i<strlen(string); i++) 
  {
    char ch = string[i];
    int result = write(dev->fd, &ch, 1);
		
    if(result != 1) 
    {
      mysleep(10000);
      const int MAXWRITERETRY = 10;
      int j=0;
      for (; j < MAXWRITERETRY; j++)
      {
//        perror("write '%c' failed for the %d time, result = %d\n", ch, j+1, myResult);
	result = write(dev->fd, &ch, 1);
	if(result == 1)
	  break;
	mysleep(10000);
      }
      if (j>=MAXWRITERETRY)
      {
	close_dev(dev);
      }
    }
    mysleep(5000);
  }
  return true;
}

char get_onebyte(t_device *dev, unsigned int sleeptime)
{
  if (dev->fd == -1)
  {
    return -1;
  }

  char  my_char = 0;
  struct timeval timeout;
  timeout.tv_sec  = 0L;
  timeout.tv_usec = sleeptime;
  fd_set my_fdset;
  FD_ZERO(&my_fdset);
  FD_SET(dev->fd, &my_fdset);
  int result = select(dev->fd+1, &my_fdset, NULL, NULL, &timeout);
  if (result == -1)
  {
    my_char = 0;
  }
  if (result != 0)
  {
    result=read(dev->fd, &my_char, 1);
    if (result != 1)
    {
      my_char=0;
    }
  }
  return my_char;
}

char *get_string(t_device *dev, char *str, unsigned int maxsize, unsigned int sleeptime) {
  char *result = str;
  
  if (dev->fd == -1) {
    return result;
  }
  
  if (maxsize <= 0) {
    return result;
  }
  
  if (result > 0) {
    unsigned int pos = -1;
    char ch = 0;
    do {
      ch = get_onebyte(dev, sleeptime);

      if (ch == '\n') continue;
      if (ch == '\r') break;
      pos++;
      result[pos] = ch;
    } while ((pos < maxsize-1) && (ch != '\0') && (ch != -1));
  }
  
  return result;
}

int reset_dev(t_device *dev, unsigned int sleeptime)
{
  if (close_dev(dev)==false)
    return -1;
  sleep(1);
  if ((dev->fd = open_dev(dev, sleeptime))==-1)
    return dev->fd;
  int ch=1;
  while (ch > 0)
  {
    ch = get_onebyte(dev, sleeptime);
  }
  return dev->fd;
}

int lock_dev(char *filename) {
   if (!filename) 
     return -1;
     
   FILE *f = fopen(filename, "w+");
   
   if (!f)
     return 0;
     
   if (fwrite("1", 1, 1, f) <= 0)
     return 0;

   fclose(f);
   return 1;
}

int unlock_dev(char *filename) {
   if (!filename)
     return -1;
     
   FILE *f = fopen(filename, "w+");
   
   if (!f)
     return 0;
     
   if (fwrite("0", 1, 1, f) <= 0)
     return 0;

   fclose(f);
   return 1;
}

bool is_devlocked(char *filename) {
  if (!filename)
    return true;
     
  FILE *f = fopen(filename, "r+");
   
  if (!f) {
    //if file does not exist :)
    unlock_dev(filename);
    return true;
  }     
  char l = '\0';
  if (fread(&l, 1, 1, f) <= 0)
    return true;

  fclose(f);

  if (l == '0')
    return false;
  else if (l == '1')
    return true;
    
  return false;
}
